<?php
/*--------------------------------------------------------------------------------------------------
    OnGetSellingUnitReservedQuantityEvent.php 2020-07-08
    Gambio GmbH
    http://www.gambio.de
    Copyright (c) 2020 Gambio GmbH
    Released under the GNU General Public License (Version 2)
    [http://www.gnu.org/licenses/gpl-2.0.html]
 -------------------------------------------------------------------------------------------------*/

declare(strict_types=1);

namespace Gambio\Shop\SellingUnit\Database\Unit\Events;

use Gambio\Shop\SellingUnit\Database\Unit\Events\Interfaces\OnGetSellingUnitReservedQuantityEventInterface;
use Gambio\Shop\SellingUnit\Unit\ValueObjects\Interfaces\QuantityInterface;
use Gambio\Shop\SellingUnit\Unit\ValueObjects\ReservedQuantity;
use Gambio\Shop\SellingUnit\Unit\ValueObjects\SellingUnitId;
use ProductDataInterface;

/**
 * Class GetProductReservedQuantityEvent
 */
class OnGetSellingUnitReservedQuantityEvent implements OnGetSellingUnitReservedQuantityEventInterface
{
    /**
     * @var SellingUnitId
     */
    protected $id;
    /**
     * @var ReservedQuantity
     */
    protected $quantity;
    /**
     * @var bool
     */
    protected $propagationStopped = false;
    /**
     * @var ProductDataInterface
     */
    protected $productData;
    /**
     * @var QuantityInterface
     */
    protected $requested;


    /**
     * GetSellingUnitReservedQuantityEvent constructor.
     * @param SellingUnitId $id
     * @param ProductDataInterface $productData
     * @param QuantityInterface $requested
     */
    public function __construct(SellingUnitId $id, ProductDataInterface $productData, QuantityInterface $requested)
    {

        $this->id = $id;
        $this->productData = $productData;
        $this->quantity = new ReservedQuantity(0,'');
        $this->requested = $requested;
    }

    /**
     * @inheritDoc
     */
    public function id(): SellingUnitId
    {
        return $this->id;
    }

    /**
     * @inheritDoc
     */
    public function isPropagationStopped(): bool
    {
        return $this->propagationStopped;
    }

    /**
     * @inheritDoc
     */
    public function quantity(): ?ReservedQuantity
    {
        return $this->quantity;
    }

    /**
     * @return QuantityInterface
     */
    public function requested(): QuantityInterface
    {
        return $this->requested;
    }

    /**
     * @inheritDoc
     */
    public function setQuantity(ReservedQuantity $reservedQuantity): void
    {
        $this->quantity = $reservedQuantity;
    }

    public function stop(): void
    {
        $this->propagationStopped = true;
    }
}